class ReporterInterface(object):

    def notify_before_console_output(self):
        pass

    def notify_after_console_output(self):
        pass

    def report_before_debugger(self, exc_info):
        pass

    def report_session_start(self, session):
        pass

    def report_session_end(self, session):
        pass

    def report_file_start(self, filename):
        pass

    def report_file_end(self, filename):
        pass

    def report_collection_start(self):
        pass

    def report_test_collected(self, all_tests, test):
        pass

    def report_collection_end(self, collected):
        pass

    def report_test_start(self, test):
        pass

    def report_test_end(self, test, result):
        if result.is_success():
            self.report_test_success(test, result)
        elif result.is_skip():
            self.report_test_skip(test, result)
        elif result.is_error():
            self.report_test_error(test, result)
        else:
            assert result.is_failure()
            self.report_test_failure(test, result)

    def report_test_success(self, test, result):
        pass

    def report_test_skip(self, test, result):
        pass

    def report_test_error(self, test, result):
        pass

    def report_test_failure(self, test, result):
        pass

    def report_test_error_added(self, test, error):
        pass

    def report_test_failure_added(self, test, error):
        pass

    def report_test_skip_added(self, test, reason):
        pass

    def report_fancy_message(self, headline, message):
        pass

    def report_message(self, message):
        pass




from .utils.suite_writer import Suite

def test_default_factory_balanced_between_methods_and_functions():
    suite = Suite()
    assert suite.num_method_tests == suite.num_function_tests == 0
    suite.add_test()
    suite.add_test()
    assert suite.num_method_tests == suite.num_function_tests == 1



from __future__ import print_function

import threading
from contextlib import contextmanager
from ..ctx import context
from ..core import metadata
from ..core.function_test import FunctionTestFactory

try:
    from IPython import embed # pylint: disable=F0401
except ImportError:
    import code
    def _interact(ns):
        code.interact(local=ns)
else:
    def _interact(ns):
        embed(user_ns=ns)

def start_interactive_shell(**namespace):
    """
    Starts an interactive shell. Uses IPython if available, else fall back
    to the native Python interpreter.

    Any keyword argument specified will be available in the shell ``globals``.
    """
    if context.g is not None:
        namespace.update(context.g.__dict__)

    _interact(namespace)

def generate_interactive_test():
    [returned] = FunctionTestFactory(start_interactive_shell).generate_tests(context.session.fixture_store)
    m = returned.__slash__ = metadata.Metadata(None, returned, '')
    m.factory_name = m.address = 'Interactive'
    return returned

@contextmanager
def notify_if_slow_context(message, slow_seconds=1):
    evt = threading.Event()
    def notifier():
        if not evt.wait(timeout=slow_seconds) and context.session is not None:
            context.session.reporter.report_message(message)
    thread = threading.Thread(target=notifier)
    thread.start()
    try:
        yield
    finally:
        evt.set()
        thread.join()



# pylint: disable-msg=W0201
import gossip
import pytest

import slash


def test_interruption(interrupted_suite, interrupted_index):
    interrupted_suite.run(expect_interruption=True)


def test_interruption_triggers_gossip(request, interrupted_suite, interrupted_test):
    test_id = {'value': None}

    @gossip.register('slash.test_interrupt')
    def skip():
        test_id['value'] = slash.test.__slash__.id

    @request.addfinalizer
    def cleanup():
        skip.gossip.unregister()

    summary = interrupted_suite.run(expect_interruption=True)
    assert test_id['value'] is not None
    for result in summary.get_all_results_for_test(interrupted_test):
        assert result.test_metadata.id == test_id['value']


def test_critical_cleanups_called(interrupted_suite, interrupted_test):
    cleanup = interrupted_test.add_deferred_event(
        'slash.add_critical_cleanup', 'critical_cleanup')
    summary = interrupted_suite.run(expect_interruption=True)
    assert cleanup in summary.events


def test_non_critical_cleanups_not_called(interrupted_suite, interrupted_test):
    cleanup = interrupted_test.add_cleanup()
    summary = interrupted_suite.run(expect_interruption=True)
    assert cleanup not in summary.events


@pytest.fixture
def interrupted_suite(suite, interrupted_index):
    for index, test in enumerate(suite):
        if index == interrupted_index:
            test.append_line('raise KeyboardInterrupt()')
            test.expect_interruption()
        elif index > interrupted_index:
            test.expect_deselect()

    return suite


@pytest.fixture
def interrupted_test(interrupted_suite, interrupted_index):
    return interrupted_suite[interrupted_index]


@pytest.fixture
def interrupted_index(suite):
    return int(len(suite) // 2)



import itertools

_ID_SEPARATOR = "_"

class IDSpace(object):
    def __init__(self, base):
        super(IDSpace, self).__init__()
        if not base.endswith(_ID_SEPARATOR):
            base += _ID_SEPARATOR
        self._allocator = (base + str(i) for i in itertools.count(1))
    def allocate(self):
        return next(self._allocator)



# pylint: disable-msg=W0201
import functools

import pytest
import slash
from slash.exceptions import NoActiveSession
from slash.runner import run_tests


def test_run_tests_fails_without_active_session():
    with pytest.raises(NoActiveSession):
        run_tests([])


def test_simple_run(suite):
    suite.run()


def test_iter_results_ordering(suite):
    for index, test in enumerate(suite):
        test.append_line('slash.context.result.data["index"] = {0}'.format(index))

    results = list(suite.run().session.results.iter_test_results())

    for index, result in enumerate(results):
        assert result.data['index'] == index


def test_failed(suite, test):
    test.when_run.fail()
    result = suite.run()[test]
    assert result.is_failure()
    assert not result.is_error()
    assert not result.is_success()


def test_error(suite, test):
    test.when_run.error()
    result = suite.run()[test]
    assert result.is_error()
    assert not result.is_failure()
    assert not result.is_success()


def test_skip(suite, test):
    test.when_run.skip()
    result = suite.run()[test]
    assert result.is_skip()
    assert not result.is_error()
    assert not result.is_failure()
    assert not result.is_success()


@pytest.mark.parametrize('failure_type', ['failure', 'error'])
def test_stop_on_fatal_exception(suite, test, remaining_tests, failure_type):
    test.append_line(
        'from slash.exception_handling import mark_exception_fatal')
    test.append_line('raise mark_exception_fatal({0}())'.format('AssertionError' if failure_type == 'failure' else 'Exception'))
    getattr(test, 'expect_{0}'.format(failure_type))()
    for remaining_test in remaining_tests:
        remaining_test.expect_not_run()

    suite.run()


@pytest.mark.parametrize('failure_type', ['error', 'failure'])
def test_stop_on_error(suite, test, test_index, failure_type):
    if failure_type == 'error':
        test.when_run.error()
    elif failure_type == 'failure':
        test.when_run.fail()
    else:
        raise NotImplementedError()  # pragma: no cover

    for index, test in enumerate(suite):
        if index > test_index:
            test.expect_not_run()

    suite.run(additional_args=['-x'])


def test_stop_on_error_unaffected_by_skips(suite, test):
    assert test is not suite[-1]
    test.when_run.skip()
    results = suite.run(additional_args=['-x'])
    assert results[suite[-1]].is_success()


def test_debug_if_needed(request, config_override, suite, test):
    test.when_run.fail()

    debugged = {'value': False}

    def _debug_if_needed(exc_info):
        debugged['value'] = True

    request.addfinalizer(functools.partial(
        setattr, slash.utils.debug, '_KNOWN_DEBUGGERS', slash.utils.debug._KNOWN_DEBUGGERS))
    slash.utils.debug._KNOWN_DEBUGGERS = [_debug_if_needed]

    config_override('debug.enabled', True)

    suite.run()

    assert debugged['value']


@pytest.fixture
def test(suite, test_index):
    return suite[test_index]


@pytest.fixture
def test_index(suite):
    return int(len(suite) // 2)


@pytest.fixture
def remaining_tests(suite, test_index):
    return suite[test_index + 1:]



import itertools
import traceback
import os
from types import FunctionType, GeneratorType
from contextlib import contextmanager

from emport import import_file
from logbook import Logger

import dessert
import sys

from .conf import config
from ._compat import iteritems, string_types
from .ctx import context
from .core.local_config import LocalConfig
from .core.runnable_test import RunnableTest
from .core.test import Test, TestTestFactory
from .core.function_test import FunctionTestFactory
from .exception_handling import handling_exceptions
from .exceptions import CannotLoadTests
from .core.runnable_test_factory import RunnableTestFactory
from .utils import add_error
from .utils.pattern_matching import Matcher

_logger = Logger(__name__)


class Loader(object):

    """
    Provides iteration interfaces to load runnable tests from various places
    """

    def __init__(self):
        super(Loader, self).__init__()
        if config.root.run.filter_string:
            self._matcher = Matcher(config.root.run.filter_string)
        else:
            self._matcher = None
        self._local_config = LocalConfig()

    def get_runnables(self, paths, sort_key=None):
        assert context.session is not None
        returned = self._collect(self._get_iterator(paths))
        if sort_key is not None:
            returned.sort(key=sort_key)
        return returned

    def _collect(self, iterator):
        returned = []
        context.reporter.report_collection_start()
        try:
            for x in iterator:
                returned.append(x)
                context.reporter.report_test_collected(returned, x)
        finally:
            context.reporter.report_collection_end(returned)

        return returned

    def _get_iterator(self, thing):
        if isinstance(thing, (list, GeneratorType, itertools.chain)):
            return itertools.chain.from_iterable(self._get_iterator(x) for x in thing)
        if isinstance(thing, string_types):
            return self._iter_test_address(thing)
        if isinstance(thing, RunnableTest):
            return [thing]

        if not isinstance(thing, RunnableTestFactory):
            thing = self._get_runnable_test_factory(thing)

        return thing.generate_tests(fixture_store=context.session.fixture_store)

    def _iter_test_address(self, address):
        if ':' in address:
            path, address_in_file = address.split(':', 1)
        else:
            path = address
            address_in_file = None

        for test in self._iter_path(path):
            if address_in_file is not None:
                if not self._address_in_file_matches(address_in_file, test):
                    continue
            yield test

    def _address_in_file_matches(self, address_in_file, test):
        if address_in_file == test.__slash__.factory_name:
            return True
        test_address_in_file = test.__slash__.address_in_file
        if address_in_file == test_address_in_file:
            return True
        if '(' in test_address_in_file:
            if address_in_file == test_address_in_file[:test_address_in_file.index('(')]:
                return True

        return False

    def _iter_path(self, path):
        return self._iter_paths([path])

    def _iter_paths(self, paths):
        paths = list(paths)
        for path in paths:
            if not os.path.exists(path):
                msg = "Path {0} could not be found".format(path)
                add_error(msg)
                raise CannotLoadTests(msg)
        for path in paths:
            for file_path in _walk(path):
                _logger.debug("Checking {0}", file_path)
                if not self._is_file_wanted(file_path):
                    _logger.debug("{0} is not wanted. Skipping...", file_path)
                    continue
                module = None
                try:
                    with handling_exceptions(context="during import"):
                        with dessert.rewrite_assertions_context():
                            module = import_file(file_path)
                except Exception as e:
                    tb_file, tb_lineno, _, _ = traceback.extract_tb(sys.exc_info()[2])[-1]
                    raise CannotLoadTests(
                        "Could not load {0!r} ({1}:{2} - {3})".format(file_path, tb_file, tb_lineno, e))
                if module is not None:
                    with self._adding_local_fixtures(file_path, module):
                        for runnable in self._iter_runnable_tests_in_module(file_path, module):
                            if self._is_excluded(runnable):
                                continue
                            yield runnable

    @contextmanager
    def _adding_local_fixtures(self, file_path, module):
        context.session.fixture_store.push_namespace()
        try:
            self._local_config.push_path(os.path.dirname(file_path))
            try:
                context.session.fixture_store.add_fixtures_from_dict(
                    self._local_config.get_dict())
                context.session.fixture_store.add_fixtures_from_dict(
                    vars(module))
                context.session.fixture_store.resolve()
                yield
            finally:
                self._local_config.pop_path()
        finally:
            context.session.fixture_store.pop_namespace()

    def _is_excluded(self, test):
        if self._matcher is None:
            return False
        return not self._matcher.matches(test.__slash__.address)

    def _is_file_wanted(self, filename):
        return filename.endswith(".py")

    def _iter_runnable_tests_in_module(self, file_path, module):
        for thing_name, thing in iteritems(vars(module)):
            if thing is RunnableTestFactory:  # probably imported directly
                continue

            factory = self._get_runnable_test_factory(thing)

            if factory is None:
                continue

            factory.set_factory_name(thing_name)
            factory.set_module_name(module.__name__)
            factory.set_filename(file_path)

            for test in factory.generate_tests(fixture_store=context.session.fixture_store):
                assert test.__slash__ is not None
                yield test

    def _get_runnable_test_factory(self, thing):

        if isinstance(thing, type) and issubclass(thing, Test):
            return TestTestFactory(thing)

        if isinstance(thing, FunctionType):
            if thing.__name__.startswith('test_'):
                return FunctionTestFactory(thing)

        return None


def _walk(p):
    if os.path.isfile(p):
        return [p]
    return (os.path.join(dirname, filename)
            for dirname, _, filenames in os.walk(p)
            for filename in filenames)



import sys

class Metadata(object):

    """Class representing the metadata associated with a test object. Generally available
    as test.__slash__
    """

    #: The index of the test in the current execution, 0-based
    test_index0 = None

    def __init__(self, factory, test, address_in_factory=None):
        super(Metadata, self).__init__()
        #: The test's unique id
        self.id = None
        #: Back reference to the test to which this metadata placeholder belongs
        self.test = test
        if factory is not None:
        #: The path to the file from which this test was loaded
            self.module_name = factory.get_module_name()
            assert self.module_name, 'Could not find module for {0}'.format(test)
            self.file_path = factory.get_filename()
            self.factory_name = factory.get_factory_name()
        else:
            self.module_name = type(test).__module__
            self.file_path = sys.modules[self.module_name].__file__
            self.factory_name = '?'
        #: Address string to identify the test inside the file from which it was loaded
        self.address_in_file = self.factory_name
        self.address_in_factory = address_in_factory
        if address_in_factory is not None:
            self.address_in_file += address_in_factory
        #: String identifying the test, to be used when logging or displaying results in the console
        #: generally it is composed of the file path and the address inside the file
        self.address = '{0}:{1}'.format(self.file_path, self.address_in_file)

    @property
    def test_index1(self):
        """Same as ``test_index0``, only 1-based
        """
        if self.test_index0 is None:
            return None
        return self.test_index0 + 1

    def __repr__(self):
        return '<{0}>'.format(self.address)


def ensure_test_metadata(test):
    returned = getattr(test, "__slash__", None)
    if returned is None:
        returned = test.__slash__ = Metadata(None, test, '')
    return returned



import os
import shutil
import tempfile
from tempfile import mkdtemp

import pytest
from slash.frontend.slash_run import _iter_suite_file_paths


def test_iter_suite_paths_files_abspaths(filename, paths):
    with open(filename, 'w') as f:
        f.write('\n'.join(paths))

    assert list(_iter_suite_file_paths([filename])) == paths

def test_iter_suite_paths_files_relpath(filename, paths):
    with open(filename, 'w') as f:
        for path in paths:
            relpath = os.path.relpath(path, os.path.dirname(filename))
            assert not os.path.isabs(relpath)
            f.write(relpath)
            f.write('\n')

    assert list(_iter_suite_file_paths([filename])) == [os.path.abspath(p) for p in paths]

@pytest.mark.parametrize('use_relpath', [True, False])
def test_files_containing_files(filename, paths, use_relpath):
    filename2 = os.path.join(os.path.dirname(filename), 'file2.txt')

    with open(filename2, 'w') as f:
        f.write('\n'.join(paths[::-1]))

    if use_relpath:
        filename2 = os.path.basename(filename2)

    with open(filename, 'w') as f:
        f.write('\n'.join(paths))
        f.write('\n')
        f.write(filename2)

    assert list(_iter_suite_file_paths([filename])) == paths + paths[::-1]


@pytest.fixture
def filename(tmpdir):
    return str(tmpdir.join('filename.txt'))

@pytest.fixture
def paths(request, tmpdir, use_relpath_for_dir):
    basenames = ['file{0}.py'.format(i) for i in range(10)]
    basenames.extend(['file100.py:SomeClass',
                      'file101.py:Someclass.test_method'])
    returned = [os.path.join(os.path.abspath(str(tmpdir)), b) for b in basenames]

    if use_relpath_for_dir:
        dirname = str(tmpdir.join('dirname'))
        os.makedirs(dirname)
    else:
        dirname = mkdtemp()
        @request.addfinalizer
        def cleanup():
            shutil.rmtree(dirname)
    returned.append(dirname)

    return returned

@pytest.fixture(params=[True, False])
def use_relpath_for_dir(request):
    return request.param



import functools
import inspect
import sys

from sentinels import NOTHING

from .._compat import reraise


def wraps(func, preserve=()):
    def decorator(new_func):
        returned = functools.wraps(func)(new_func)
        returned.__wraps__ = func
        return returned
    for p in preserve:
        orig = getattr(func, p, NOTHING)
        if orig is not NOTHING:
            setattr(decorator, p, orig)
    return decorator

def get_underlying_func(func):
    while True:
        underlying = getattr(func, "__wraps__", None)
        if underlying is None:
            return func
        func = underlying

def getargspec(func):
    return inspect.getargspec(get_underlying_func(func))


def call_all_raise_first(_funcs, *args, **kwargs):
    exc_info = None
    for func in _funcs:
        try:
            func(*args, **kwargs)
        except Exception: # pylint: disable=broad-except
            exc_info = sys.exc_info()
    if exc_info is not None:
        reraise(*exc_info)



import logbook

import pytest
from slash.utils.deprecation import deprecated, get_no_deprecations_context


def test_deprecated_func_called(capture):
    assert deprecated_func(1, 2) == 3


def test_deprecation_message(capture):
    deprecated_func(1, 2)

    [record] = capture.records
    assert 'deprecated' in record.message
    assert 'deprecated_func' in record.message


def test_deprecation_with_message(capture):

    @deprecated('use something else instead', since='1.0.0')
    def func(a, b):
        return a + b

    func(1, 2)

    [record] = capture.records
    assert 'use something else instead' in record.message
    assert 'func is deprecated' in record.message


def test_no_deprecations(capture):

    @deprecated('msg', since='1.0.0')
    def func(a, b):
        return a + b

    with get_no_deprecations_context():
        assert func(1, 2) == 3
    assert not capture.records


def test_deprecations_different_sources(capture):

    def f():
        deprecated_func(1, 2)

    def g():
        deprecated_func(1, 2)

    f()
    g()
    assert len(capture.records) == 2


def test_deprecations_same_sources(capture):

    def f():
        deprecated_func(1, 2)

    f()
    f()
    assert len(capture.records) == 1


def test_deprecatd_docstring():

    message = 'Use something else instead'

    @deprecated(since='1.0.0')
    def some_func():
        """This is a function
        """

    @deprecated(message, since='1.0.0')
    def other_func():
        """This is another function
        """

    assert '.. deprecated:: 1.0.0' in some_func.__doc__
    assert '.. deprecated:: 1.0.0\n   {0}'.format(message) in other_func.__doc__


@pytest.fixture
def capture(request):
    handler = logbook.TestHandler(level=logbook.WARNING)
    handler.push_application()

    @request.addfinalizer
    def pop():
        handler.pop_application()
    return handler


@deprecated(since='1.0.0')
def deprecated_func(a, b):
    return a + b



import gossip

from .conf import config
from .utils.deprecation import deprecated


def _deprecated_to_gossip(func):
    return deprecated(since="0.6.0", message="Use gossip instead")(func)

def _define(hook_name, **kwargs):
    hook = gossip.define("slash.{0}".format(hook_name), **kwargs)
    globals()[hook_name] = hook
    return hook

_define('session_start', doc="Called right after session starts")
_define('session_end', doc="Called right before the session ends, regardless of the reason for termination")

_define('after_session_start', doc="Second entry point for session start, useful for plugins relying on other plugins' session_start routine")

_define('test_interrupt', doc="Called when a test is interrupted by a KeyboardInterrupt or other similar means")
_define('test_start', doc="Called right after a test starts")
_define('test_end', doc="Called right before a test ends, regardless of the reason for termination")
_define('before_test_cleanups', doc="Called right before a test cleanups are executed")
_define('test_success', doc="Called on test success")
_define('test_error', doc="Called on test error")
_define('test_failure', doc="Called on test failure")
_define('test_skip', doc="Called on test skip", arg_names=("reason",))

_define('error_added', doc='Called when an error is added to a result (either test result or global)', arg_names=('error', 'result'))

_define('result_summary', doc="Called at the end of the execution, when printing results")

_define('exception_caught_before_debugger',
        doc="Called whenever an exception is caught, but a debugger hasn't been entered yet")

_define('exception_caught_after_debugger',
        doc="Called whenever an exception is caught, and a debugger has already been run")

_slash_group = gossip.get_group('slash')
_slash_group.set_strict()
_slash_group.set_exception_policy(gossip.RaiseDefer())

@gossip.register('gossip.on_handler_exception') # pylint: disable=unused-argument
def debugger(handler, exception, hook): # pylint: disable=unused-argument
    from .exception_handling import handle_exception

    if hook.group is _slash_group and config.root.debug.debug_hook_handlers:
        handle_exception(exception)

@_deprecated_to_gossip
def add_custom_hook(hook_name):
    """
    Adds an additional hook to the set of available hooks
    """
    return _define(hook_name)

@_deprecated_to_gossip
def ensure_custom_hook(hook_name):
    """
    Like :func:`.add_custom_hook`, only forgives if the hook already exists
    """
    try:
        return gossip.get_hook("slash.{0}".format(hook_name))
    except LookupError:
        return _define(hook_name)

@_deprecated_to_gossip
def remove_custom_hook(hook_name):
    """
    Removes a hook from the set of available hooks
    """
    gossip.get_hook("slash.{0}".format(hook_name)).undefine()
    globals().pop(hook_name)

@_deprecated_to_gossip
def get_custom_hook_names():
    """
    Retrieves the names of all custom hooks currently installed
    """
    raise NotImplementedError()  # pragma: no cover

@_deprecated_to_gossip
def get_all_hooks():
    return [
        (hook.name, hook)
        for hook in gossip.get_group('slash').get_hooks()]

@_deprecated_to_gossip
def get_hook_by_name(hook_name):
    """
    Returns a hook (if exists) by its name, otherwise returns None
    """
    return gossip.get_hook('slash.{0}'.format(hook_name))



from __future__ import print_function

import os
import sys

import pytest
from slash import config, site
from slash._compat import StringIO
from slash.frontend import slash_run
from slash.frontend.main import main_entry_point

from .utils import no_op, NullFile, TestCase


class SlashRunTestBase(TestCase):

    def setUp(self):
        super(SlashRunTestBase, self).setUp()
        self.override_config("run.session_state_path",
                             os.path.join(self.get_new_path(), "session_data"))


class MissingFilesTest(SlashRunTestBase):

    def test_slash_run_fails_fast_for_missing_files(self):
        result = slash_run.slash_run(
            ["/non/existing/path"], report_stream=NullFile())
        self.assertNotEquals(
            result, 0, "slash run unexpectedly succeeded for a missing path")


class ArgumentParsingTest(TestCase):

    def setUp(self):
        super(ArgumentParsingTest, self).setUp()
        self.stderr = StringIO()
        self.stdout = StringIO()
        self.addCleanup(setattr, sys, "stderr", sys.stderr)
        self.forge.replace_with(sys, "stderr", self.stderr)
        self.addCleanup(setattr, sys, "stdout", sys.stdout)
        self.forge.replace_with(sys, "stdout", self.stdout)

    callback_success = False

    def _collect_tests_stub(self, app, args, test_sort_key):
        self.assertTrue(config.root.debug.enabled)
        self.assertEquals(app.args.positionals, ["test1.py", "test2.py"])
        # this must be last to make sure the stub ran successfully
        self.callback_success = True
        return []

    def test_interspersed_positional_arguments(self):

        self.forge.replace_with(
            slash_run, "_collect_tests", self._collect_tests_stub)
        self.forge.replace_with(
            sys, "argv", "/path/to/slash run -vv test1.py -x test2.py --pdb".split())
        with self.assertRaises(SystemExit) as caught:
            main_entry_point()
        self.assertTrue(self.callback_success)
        if isinstance(caught.exception, int):
            # python 2.6
            code = caught.exception
        else:
            code = caught.exception.code
        self.assertEquals(code, 0)


class SlashHelpTest(ArgumentParsingTest):

    def _fake_execute(self, argv):
        prev_argv = list(sys.argv)
        sys.argv = argv[:]
        try:
            main_entry_point()
        finally:
            sys.argv = prev_argv

    def test_slash_run_help(self):

        with self.assertRaises(SystemExit):
            self._fake_execute(["slash", "run", "-h"])

        self.assertTrue(self.stdout.getvalue().startswith("usage: slash run "))
        self.assertIn("TEST [TEST ", self.stdout.getvalue())

    def test_slash_help(self):

        with self.assertRaises(SystemExit):
            self._fake_execute(["slash", "-h"])

        self.assertTrue(self.stdout.getvalue().startswith(
            "usage: slash command..."), self.stdout.getvalue())


def test_slash_run_directory_success(suite):
    assert suite.run().exit_code == 0


def test_slash_run_default_directory(config_override, suite, suite_path):
    config_override("run.default_sources", [suite_path])
    suite.run(args=[], commit=False)


def test_slash_run_success_if_skips(suite):
    suite[1].when_run.skip()
    assert suite.run().exit_code == 0


def test_slash_run_from_file(tmpdir, suite):

    for i in range(20):
        suite.add_test()

    suite_path = suite.commit()

    assert len(suite.files) > 2
    file1 = suite.files[0]
    file2 = suite.files[-1]

    deselected = [t for t in suite if t.file not in (file1, file2)]
    assert deselected
    for d in deselected:
        d.expect_deselect()

    assert os.path.isdir(suite_path)

    filename1 = str(tmpdir.join("file1.txt"))
    filename2 = str(tmpdir.join("file2.txt"))

    for suite_file, filename in zip((file1, file2), (filename1, filename2)):
        with open(filename, "w") as f:
            print("# this is a comment", file=f)
            f_name = os.path.join(suite_path, suite_file.get_relative_path())
            assert os.path.isdir(suite_path)
            assert os.path.exists(f_name)
            print(f_name, file=f)

    suite.run(args=["-f", filename1, "-f", filename2], commit=False)


@pytest.mark.parametrize('failure_type', ['fail', 'error'])
def test_slash_run_directory_failure(suite, failure_type):
    getattr(suite[1].when_run, failure_type)()
    path = suite.commit()
    assert suite.run().exit_code != 0


def test_slash_run_specific_file(suite):

    for i in range(5):
        suite.add_test()
    suite_path = suite.commit()

    assert len(suite.files) > 1

    file = suite.files[1]

    deselected = [t for t in suite if t.file != file]
    assert deselected
    for t in deselected:
        t.expect_deselect()

    suite.run(args=[os.path.join(suite_path, file.get_relative_path())], commit=False)


@pytest.fixture(autouse=True)
def no_site_load(forge):
    forge.replace_with(site, 'load', no_op)


@pytest.fixture
def suite_path(suite):
    returned = suite.commit()
    assert os.path.isdir(returned)
    return returned



from .._compat import xrange

from .fixtures.parameters import bound_parametrizations_context
from .requirements import get_requirements
from .runnable_test import RunnableTest
from .runnable_test_factory import RunnableTestFactory


class FunctionTest(RunnableTest):

    def __init__(self, function, fixture_store, fixture_namespace, variation):
        super(FunctionTest, self).__init__()
        self._func = function
        self._fixture_store = fixture_store
        self._fixture_namespace = fixture_namespace
        self._variation = variation

    def run(self):
        with bound_parametrizations_context(self._variation):
            self._fixture_store.activate_autouse_fixtures_in_namespace(namespace=self._fixture_namespace)
            self._fixture_store.call_with_fixtures(
                self._func, namespace=self._fixture_namespace,
                is_method=False
            )

    def get_requirements(self):
        return get_requirements(self._func)

    def get_required_fixture_objects(self):
        return self._fixture_store.get_required_fixture_objects(self._func, namespace=self._fixture_namespace, is_method=False)


class FunctionTestFactory(RunnableTestFactory):

    def __init__(self, func):
        super(FunctionTestFactory, self).__init__(func)
        self.func = func

    def _generate_tests(self, fixture_store):
        namespace = fixture_store.get_current_namespace()
        for variation in fixture_store.iter_parametrization_variations(funcs=[self.func]):
            address = '({0})'.format(variation.representation) if variation else None
            for _ in xrange(self._get_num_repetitions(self.func)):
                yield address, FunctionTest(self.func, fixture_store, namespace, variation)



from .utils.suite_writer import Suite

from contextlib import contextmanager

import pytest


def test_expect_failure_not_met(suite, test):
    test.expect_failure()
    with _raises_assertion('Test did not fail as expected'):
        suite.run()

def test_expect_error_not_met(suite, test):
    test.expect_error()
    with _raises_assertion('Test did not issue error as expected'):
        suite.run()



@contextmanager
def _raises_assertion(msg):
    with pytest.raises(AssertionError) as caught:
        yield
    assert str(caught.value) == msg

@pytest.fixture
def test(suite):
    return suite[len(suite) // 2]

@pytest.fixture
def suite():
    s = Suite()
    for i in range(10):
        s.add_test()
    return s



import functools
import sys
import threading
from contextlib import contextmanager

import logbook

from .._compat import string_types

_deprecation_logger = logbook.Logger("slash.deprecation")
_deprecation_locations = set()

class _Local(threading.local):
    enabled = True

_local = _Local()


@contextmanager
def get_no_deprecations_context():
    """Disables deprecation messages temporarily
    """
    prev_enabled = _local.enabled
    _local.enabled = False
    try:
        yield
    finally:
        _local.enabled = prev_enabled


def deprecated(func=None, message=None, since=None):
    """Marks the specified function as deprecated, and emits a warning when it's called
    """
    if isinstance(func, string_types):
        assert message is None
        message = func
        func = None

    if func is None:
        return functools.partial(deprecated, message=message, since=since)

    if not since:
        raise ValueError(
            "Must provide deprecation version via the 'since' parameter")

    @functools.wraps(func)
    def new_func(*args, **kwargs):
        if _local.enabled:
            caller_location = _get_caller_location()
            if caller_location not in _deprecation_locations:
                warning = "{func.__module__}.{func.__name__} is deprecated.".format(
                    func=func)
                if message is not None:
                    warning += " {0}".format(message)
                _deprecation_logger.warning(warning)
                _deprecation_locations.add(caller_location)
        return func(*args, **kwargs)

    if new_func.__doc__:  # pylint: disable=no-member
        new_func.__doc__ += "\n.. deprecated:: {0}\n".format(since)  # pylint: disable=no-member
        if message:
            new_func.__doc__ += "   {0}".format(message)  # pylint: disable=no-member

    return new_func


def _get_caller_location(stack_climb=2):
    frame = sys._getframe(stack_climb)  # pylint: disable=protected-access
    try:
        return (frame.f_code.co_name, frame.f_lineno)
    finally:
        del frame



import pytest

import slash
from slash.core.function_test import FunctionTestFactory
from slash.core.test import TestTestFactory

def test_factory_name(factory, expected_factory_name):
    assert factory.get_factory_name() == expected_factory_name

def test_module_name(factory, expected_module_name):
    assert factory.get_module_name() == expected_module_name

def test_filename(factory, expected_filename):
    assert factory.get_filename() == expected_filename

################################################################################

@pytest.fixture
def factory(explicit, factory_class, factory_param):
    returned = factory_class(factory_param)
    return returned

@pytest.fixture
def expected_factory_name(factory, explicit, factory_param):
    if explicit:
        returned = 'SomeFactoryNameHere'
        factory.set_factory_name(returned)
        return returned
    else:
        return factory_param.__name__

@pytest.fixture
def expected_filename(factory, explicit):
    if explicit:
        returned = 'some_nonexisting_file.py'
        factory.set_filename(returned)
        return returned
    else:
        returned = __file__
        if returned.endswith('.pyc'):
            returned = returned[:-1]
        return returned

@pytest.fixture
def expected_module_name(factory, explicit):
    if explicit:
        returned = 'some_module'
        factory.set_module_name(returned)
        return returned
    else:
        returned = __name__
        return returned


@pytest.fixture(params=[True, False])
def explicit(request):
    return request.param

@pytest.fixture(params=[FunctionTestFactory, TestTestFactory])
def factory_class(request):
    return request.param

@pytest.fixture
def factory_param(factory_class):
    if factory_class is FunctionTestFactory:
        def test_something():
            pass

        return test_something
    elif factory_class is TestTestFactory:

        class ExampleTest(slash.Test):

            def test_something(self):
                pass

        return ExampleTest

    else:
        raise NotImplementedError("Unknown factory class")



from .utils import TestCase
from slash import Session

class TestIDSpace(TestCase):
    def test_ids_are_unique(self):
        ids = []
        for _ in range(2):
            with Session() as session:
                ids.append(session.id)
                ids.append(session.id_space.allocate())
        self.assertEquals(len(ids), len(set(ids)), "IDs are not unique")



from __future__ import print_function

import doctest
import os

import pytest
import slash

_HERE = os.path.abspath(os.path.dirname(__file__))
_DOCS_ROOT = os.path.abspath(os.path.join(_HERE, "..", "doc"))


def test_sphinx_doctest(doctest_path):
    globs = {'print_function': print_function, 'slash': slash}
    result = doctest.testfile(doctest_path, module_relative=False, globs=globs)
    assert not result.failed

assert os.path.exists(_DOCS_ROOT)
_DOCTEST_PATHS = list(os.path.join(path, filename)
                      for path, _, filenames in os.walk(_DOCS_ROOT)
                      for filename in filenames
                      if filename.endswith(".rst"))
_README_PATH = os.path.join(_HERE, '..', 'README.md')


@pytest.fixture(params=_DOCTEST_PATHS + [_README_PATH])
def doctest_path(request):
    return request.param



from contextlib import contextmanager

from ..._compat import itervalues, iteritems
from .fixture_base import FixtureBase
from .utils import FixtureInfo, get_scope_by_name
from ...utils.python import wraps

_PARAM_INFO_ATTR_NAME = '__slash_parametrize__'


def parametrize(parameter_name, values):
    """Decorator to create multiple test cases out of a single function or module, where the cases vary by the value of ``parameter_name``, as iterated through ``values``.
    """

    def decorator(func):

        params = getattr(func, _PARAM_INFO_ATTR_NAME, None)
        if params is None:
            params = ParameterizationInfo()

            @wraps(func, preserve=['__slash_fixture__'])
            def new_func(*args, **kwargs):
                # for better debugging. _current_variation gets set to None on context exit
                variation = _current_variation
                for fixture in params.get_parametrization_fixtures():
                    if fixture.name not in kwargs:
                        assert variation is not None, 'Not called in parametrization context'
                        if variation.has_value_for_fixture_id(fixture.info.id):
                            kwargs[fixture.name] = variation.get_fixture_value(fixture.info.id)
                return func(*args, **kwargs)
            setattr(new_func, _PARAM_INFO_ATTR_NAME, params)
            returned = new_func
        else:
            returned = func

        params.add_options(parameter_name, values)
        return returned

    return decorator


def iterate(**kwargs):

    def decorator(func):
        for name, options in iteritems(kwargs):
            func = parametrize(name, options)(func)
        return func
    return decorator

def toggle(param_name):
    """A shortcut for :func:`slash.parametrize(param_name, [True, False]) <slash.parametrize>`

    .. note:: Also available for import as slash.parameters.toggle
    """
    return parametrize(param_name, (True, False))


_current_variation = None


@contextmanager
def bound_parametrizations_context(variation):
    global _current_variation  # pylint: disable=global-statement
    assert _current_variation is None
    _current_variation = variation
    try:
        yield
    finally:
        _current_variation = None


def get_parametrization_fixtures(func):
    param_info = getattr(func, _PARAM_INFO_ATTR_NAME, None)
    if param_info is None:
        return []
    return param_info.get_parametrization_fixtures()


class ParameterizationInfo(object):

    def __init__(self):
        super(ParameterizationInfo, self).__init__()
        self._params = {}
        self._fixtures = {}

    def add_options(self, param_name, options):
        assert param_name not in self._params

        self._params[param_name] = list(options)
        self._fixtures[param_name] = Parametrization(
            param_name, self._params[param_name])

    def get_parametrization_fixtures(self):
        return list(itervalues(self._fixtures))

class Parametrization(FixtureBase):

    def __init__(self, name, values):
        super(Parametrization, self).__init__()
        self.name = name
        self.info = FixtureInfo(name=name)
        self.scope = get_scope_by_name('test')
        self.values = list(values)

    def get_value(self, kwargs, active_fixture):
        raise NotImplementedError()  # pragma: no cover

    def get_variations(self):
        raise NotImplementedError() # pragma: no cover

    def _resolve(self, store):
        return {}



import sys

from confetti import Config
from slash import plugins
from slash._compat import cStringIO
from slash.plugins import PluginInterface
from slash.utils import cli_utils, conf_utils

from .utils import TestCase


class OutputCaptureTest(TestCase):
    def setUp(self):
        super(OutputCaptureTest, self).setUp()
        self.stderr = cStringIO()
        self.forge.replace_with(sys, "stderr", self.stderr)
        self.stdout = cStringIO()
        self.forge.replace_with(sys, "stdout", self.stdout)

class ArgumentParsingTest(OutputCaptureTest):
    def setUp(self):
        super(ArgumentParsingTest, self).setUp()
        self.config = Config({
            "a": {"a1": {"flag1": True // conf_utils.Cmdline(off="--no-flag1")}},
            "b": {"b1": {"flag2": False // conf_utils.Cmdline(on="--flag2")}},
            "string_value" : "",
            "int_value" : 0 // conf_utils.Cmdline(increase="--increase", decrease="--decrease"),
            "arg_value": "" // conf_utils.Cmdline(arg="--arg-value"),
            "list": ["existing_item"] // conf_utils.Cmdline(append="--append"),
            })

    def _cli(self, argv):
        return cli_utils.get_cli_environment_context(argv=argv, config=self.config)

    def test_config_arg_flag(self):
        with self._cli(["--arg-value=x"]):
            self.assertEquals(self.config["arg_value"], "x")

    def test_config_increase(self):
        with self._cli(["--increase"]):
            self.assertEquals(self.config["int_value"], 1)

    def test_config_decrease(self):
        with self._cli(["--decrease"]):
            self.assertEquals(self.config["int_value"], -1)


    def test_config_off_flag(self):
        with self._cli(["--no-flag1"]):
            self.assertFalse(self.config["a"]["a1"]["flag1"])
        self.assertTrue(self.config["a"]["a1"]["flag1"])

    def test_config_on_flag(self):
        with self._cli(["--flag2"]):
            self.assertTrue(self.config["b"]["b1"]["flag2"])
        self.assertFalse(self.config["b"]["b1"]["flag2"])

    def test_config_append_flag(self):
        with self._cli(["--append", "a", "--append=b"]):
            self.assertEquals(self.config["list"], ["existing_item", "a", "b"])

    def test_config_assign_flag(self):
        with self._cli(["-o", "string_value=hello", "-o", "int_value=666"]):
            self.assertEquals(self.config.root.string_value, "hello")
            self.assertEquals(self.config.root.int_value, 666)

    def test_config_assign_wrong_path_restores_config(self):
        "Given a bad override, the get_parsed_config_args_context() should leave the configuration untouched"
        with self.assertRaises(SystemExit) as caught:
            with self._cli(["-o", "string_value=bla", "-o", "int_value=hello"]):
                pass
        self.assertNotEquals(caught.exception.code, 0)
        self.assertEquals(self.config.root.int_value, 0)
        self.assertEquals(self.config.root.string_value, "")

class PluginCommandLineArgumentsTest(OutputCaptureTest):
    def setUp(self):
        super(PluginCommandLineArgumentsTest, self).setUp()
        self.plugin = SampleCommandLinePlugin()
        plugins.manager.install(self.plugin)
        self.addCleanup(plugins.manager.uninstall, self.plugin)
    def test_arguments_are_not_parsed_if_not_activated(self):
        args = ["--start-session-option", "2"]
        with self.assertRaises(SystemExit):
            with cli_utils.get_cli_environment_context(argv=args):
                pass
    def test_activation(self):
        with cli_utils.get_cli_environment_context(argv=["--with-sample-plugin"]):
            self.assertIn(self.plugin.get_name(), plugins.manager.get_active_plugins(), "plugin was not activated")

    def test_deactivation(self):
        plugins.manager.activate(self.plugin)
        with cli_utils.get_cli_environment_context(argv=["--without-sample-plugin"]):
            self.assertNotIn(self.plugin.get_name(), plugins.manager.get_active_plugins())
        self.assertIn(self.plugin.get_name(), plugins.manager.get_active_plugins())

    def test_argument_passing(self):
        with cli_utils.get_cli_environment_context(argv=["--with-sample-plugin", "--plugin-option", "value"]):
            self.assertEquals(self.plugin.cmdline_param, "value")

    def test_help_shows_available_plugins(self):
        with self.assertRaises(SystemExit):
            with cli_utils.get_cli_environment_context(argv=["-h"]):
                pass
        output = self.stdout.getvalue()
        self.assertIn("--with-sample-plugin", output)

class SampleCommandLinePlugin(PluginInterface):
    def get_name(self):
        return "sample-plugin"
    def configure_argument_parser(self, parser):
        parser.add_argument("--plugin-option")
    def configure_from_parsed_args(self, args):
        self.cmdline_param = args.plugin_option



from .utils import TestCase
import itertools
import os
import sys

_module_name_generator = ("custom_module_{0}".format(x) for x in itertools.count())

class ExtHookTest(TestCase):
    def setUp(self):
        super(ExtHookTest, self).setUp()
        self.path = self.get_new_path()
        self.addCleanup(setattr, sys, "path", list(sys.path))
        sys.path.insert(0, self.path)
        self.expected_value = 31337
        self.module_name = next(_module_name_generator)

        with open(os.path.join(self.path, "slash_{0}.py".format(self.module_name)), "w") as f:
            f.write("value = {0!r}".format(self.expected_value))

    def test_ext_hook_import(self):
        module = __import__("slash.ext.{0}".format(self.module_name), fromlist=[''])
        self.assertEquals(module.value, self.expected_value)

    def test_ext_hook_import_nonexistent(self):
        with self.assertRaises(ImportError):
            from slash.ext import nonexistent




class SlashException(Exception):

    @classmethod
    def throw(cls, *args, **kwargs):
        raise cls(*args, **kwargs)


class NoActiveSession(SlashException):
    pass


class CannotLoadTests(SlashException):
    pass


class FixtureException(CannotLoadTests):
    pass


class CyclicFixtureDependency(FixtureException):
    pass


class UnresolvedFixtureStore(FixtureException):
    pass


class UnknownFixtures(FixtureException):
    pass


class InvalidFixtureScope(FixtureException):
    pass


class TestFailed(AssertionError):

    """
    This exception class distinguishes actual test failures (mostly assertion errors,
    but possibly other conditions as well) from regular asserts.

    This is important, since regular code that is tested can use asserts, and that
    should not be considered a test failure (but rather a code failure)
    """
    pass


class SkipTest(SlashException):

    """
    This exception should be raised in order to interrupt the execution of the currently running test, marking
    it as skipped
    """

    def __init__(self, reason="Test skipped"):
        super(SkipTest, self).__init__(reason)
        self.reason = reason



from slash._compat import cStringIO
from slash.utils.formatter import Formatter
from slash.utils.color_string import ColorString
from .utils import TestCase


class FormatterTest(TestCase):

    def setUp(self):
        super(FormatterTest, self).setUp()
        self.buff = cStringIO()
        self.f = Formatter(self.buff)

    def assertOutput(self, v):
        self.assertEquals(self.buff.getvalue(), v)

    def test_write_non_strings(self):
        class MyObject(object):

            def __repr__(self):
                return 'repr'

            def __str__(self):
                return 'str'
        self.f.write("1:")
        self.f.write(MyObject())
        self.f.write("2:")
        self.f.writeln(MyObject())
        self.assertOutput("1:str2:str\n")

    def test_write_writeln(self):
        self.f.writeln("hello")
        self.assertOutput("hello\n")
        self.f.write("a")
        self.f.write("bcd")
        self.assertOutput("hello\nabcd")

    def test_indent_write_writeln(self):
        with self.f.indented():
            self.f.writeln("hello")
            self.assertOutput(" hello\n")
            self.f.write("a")
            self.f.write("bcd")
            self.f.writeln()
            self.assertOutput(" hello\n abcd\n")

    def test_empty_writeln(self):
        self.f.writeln('a')
        self.f.writeln()
        self.f.writeln()
        self.f.writeln('b')
        self.assertOutput('a\n\n\nb\n')

    def test_multiline_write(self):
        self.f.writeln('a\nb')
        with self.f.indented(3, string='.'):
            self.f.writeln('c\n d\ne')
        self.f.writeln('f')
        self.assertOutput("a\nb\n...c\n... d\n...e\nf\n")

    def test_indentation(self):
        self.f.writeln("begin")
        with self.f.indented(3):
            self.f.writeln("a")
            self.f.writeln("b")
        self.f.writeln("end")
        self.assertOutput("begin\n   a\n   b\nend\n")

    def test_indentation_different_char_through_ctor(self):
        self._test__indentation_different_char(True)

    def test_indentation_different_char_not_through_ctor(self):
        self._test__indentation_different_char(False)

    def _test__indentation_different_char(self, through_constructor):
        if through_constructor:
            self.f = Formatter(self.buff, indentation_string='*')
            indenter = self.f.indented
        else:
            indenter = lambda indentation: self.f.indented(
                indentation, string='*')
        self.f.writeln("a")
        with indenter(2):
            self.f.writeln('b')
        self.f.writeln('c')
        self.assertOutput('a\n**b\nc\n')


class FormatterColorTest(TestCase):

    def setUp(self):
        super(FormatterColorTest, self).setUp()
        self.buff = self.forge.create_wildcard_mock()
        self.buff.write("").whenever()  # indentation of 0

    def test_color_string_tty(self):
        self.buff.isatty().whenever().and_return(True)
        s = ColorString("hello", "red")
        self.buff.write(s.get_colored())
        self.forge.replay()
        Formatter(self.buff).write(s)

    def test_color_string_not_tty(self):
        self.buff.isatty().whenever().and_return(False)
        s = ColorString("hello", "red")
        self.buff.write("hello")
        self.forge.replay()
        Formatter(self.buff).write(s)



import pytest

from .utils.suite_writer import Suite

def test_fixture(suite):
    f = suite.slashconf.add_fixture()
    test = suite[1]
    test.depend_on_fixture(f)
    result = suite.run()
    result.events.assert_consecutive([
        ('fixture_start', f.id),
        ('test_start', test.id),
        ('test_end', test.id),
        ('fixture_end', f.id),
        ])

def test_parametrized_fixture():
    suite = Suite()
    test = suite.add_test()
    f1 = suite.slashconf.add_fixture()
    p = f1.add_parameter()
    test.depend_on_fixture(f1)
    summary = suite.run()
    assert summary.ok()
    assert len(summary.session.results) == len(p.values)

def test_regular_test_parametrization(suite, test):
    p = test.add_parameter()
    res = suite.run()

@pytest.fixture
def test(suite, is_method):
    if is_method:
        return suite.method_tests[0]
    return suite.function_tests[0]

@pytest.fixture(params=[True, False])
def is_method(request):
    return request.param

@pytest.fixture
def suite():
    s = Suite()
    for i in range(10):
        s.add_test()
    return s



from contextlib import contextmanager

from .function import CodeElement
from .element import Element
from .method_test import MethodTest
from .test_container import TestContainer


class Class(TestContainer, CodeElement, Element):

    def __init__(self, suite, file):
        super(Class, self).__init__(suite)
        self._decorators = []
        self.file = file
        self.suite = file.suite
        self.name = 'Test{0}'.format(self.id)

    def add_decorator(self, decorator_string):
        self._decorators.append(decorator_string)

    def add_method_test(self):
        returned = MethodTest(self.suite, self)
        self._tests.append(returned)
        self.suite.notify_test_added(returned)
        return returned

    @contextmanager
    def _body_context(self, code_formatter):
        for d in self._decorators:
            code_formatter.write('@')
            code_formatter.writeln(d)
        code_formatter.writeln('class {0}(slash.Test):'.format(self.name))
        with code_formatter.indented():
            yield

    def _write_body(self, code_formatter):
        super(Class, self)._write_body(code_formatter)
        for test in self._tests:
            test.write(code_formatter)



from __future__ import print_function

import argparse
import inspect
import os
import sys
from functools import partial

import colorama
import slash
from slash.utils.python import get_underlying_func

_COLOR_RESET = colorama.Fore.RESET + colorama.Back.RESET + colorama.Style.RESET_ALL  # pylint: disable=no-member


def _styler(style):
    return lambda s: '{0}{1}{2}'.format(style, s, _COLOR_RESET)

UNDERLINED = '\x1b[4m'

_heading_style = _styler(colorama.Fore.MAGENTA + colorama.Style.BRIGHT + UNDERLINED)  # pylint: disable=no-member
_title_style = _styler(colorama.Fore.WHITE + colorama.Style.BRIGHT)  # pylint: disable=no-member
_unused_style = _styler(colorama.Fore.YELLOW)  # pylint: disable=no-member
_doc_style = _styler(colorama.Fore.GREEN + colorama.Style.BRIGHT)  # pylint: disable=no-member


def _parse_args(args):
    parser = argparse.ArgumentParser('slash list [options] PATH...')
    parser.add_argument('--only-fixtures', dest='only', action='store_const', const='fixtures', default=None)
    parser.add_argument('--only-tests', dest='only', action='store_const', const='tests', default=None)
    parser.add_argument('paths', nargs='+', default=['.'])
    return parser.parse_args(args)


def slash_list(args, report_stream=sys.stdout):
    _print = partial(print, file=report_stream)

    args = _parse_args(args)

    with slash.Session() as session:
        slash.site.load()
        loader = slash.loader.Loader()
        runnables = loader.get_runnables(args.paths)
        used_fixtures = set()
        for test in runnables:
            used_fixtures.update(test.get_required_fixture_objects())

        if args.only in (None, 'fixtures'):
            _report_fixtures(args, session, _print, used_fixtures)

        if args.only in (None, 'tests'):
            _report_tests(args, runnables, _print)


def _report_tests(args, runnables, printer):
    if not args.only:
        printer(_heading_style('Tests'))
    for runnable in runnables:
        printer(_title_style(runnable.__slash__.address))


def _report_fixtures(args, session, printer, used_fixtures):
    if not args.only:
        printer(_heading_style('Fixtures'))
    for fixture in session.fixture_store:
        if not hasattr(fixture, 'fixture_func'):
            continue

        fixture_func = get_underlying_func(fixture.fixture_func)
        doc = inspect.cleandoc(fixture_func.__doc__) if fixture_func.__doc__ else ''

        unused_string = '' if fixture in used_fixtures else ' (Unused)'

        printer(_title_style('{0}{1}'.format(fixture.info.name, unused_string)))
        if doc:
            for line in (_doc_style(doc)).split('\n'):
                printer('    {0}'.format(line))

        printer('    Source: {0}:{1}'.format(
            os.path.relpath(inspect.getsourcefile(fixture_func), args.paths[0]),
            inspect.getsourcelines(fixture_func)[1]))
        printer('\n')



from uuid import uuid1

import pytest
from slash._compat import iteritems, OrderedDict
from slash.core.local_config import LocalConfig


def test_local_conf(local_conf, expected_dict):
    conf = local_conf.get_dict()
    for key, value in iteritems(expected_dict):
        assert [value] == conf[key]


def test_local_conf_loading_multiple_times(local_conf_dir, expected_dict):
    items = list(iteritems(expected_dict))
    local_conf = LocalConfig()

    local_conf.push_path(str(local_conf_dir.join('..')))
    var_name, var_value = items[0]
    var = local_conf.get_dict()[var_name]
    assert var == [var_value]

    local_conf.push_path(str(local_conf_dir))

    assert var is local_conf.get_dict()[var_name]


def test_local_conf_nonexistent_dir():
    with pytest.raises(RuntimeError):
        LocalConfig().push_path('/nonexistent/dir')


@pytest.fixture
def local_conf(local_conf_dir):
    returned = LocalConfig()
    returned.push_path(str(local_conf_dir))
    return returned


@pytest.fixture
def local_conf_dir(tmpdir, expected_dict):
    returned = tmpdir
    for i, (key, value) in enumerate(iteritems(expected_dict)):
        returned = returned.join('subdir{0}'.format(i))
        returned.mkdir()

        with returned.join('slashconf.py').open('w') as f:
            f.write('{0} = [{1!r}]'.format(key, value))
    return returned


@pytest.fixture
def expected_dict():
    return OrderedDict(('key_{0}'.format(i), 'value_{0}_{1}'.format(i, uuid1())) for i in range(10))



from .reporting.null_reporter import NullReporter

__all__ = ["context", "session", "test", "test_id", "g", "internal_globals"]


class GlobalStorage(object):
    pass


class Context(object):
    session = test = test_id = result = None

    def __init__(self):
        super(Context, self).__init__()
        self.g = GlobalStorage()
        self.internal_globals = GlobalStorage()

    @property
    def test_filename(self):
        return self._get_test_address_field("file_path")

    @property
    def session_id(self):
        s = self.session
        if s is None:
            return None
        return s.id

    @property
    def reporter(self):
        if self.session is None:
            return NullReporter()
        return self.session.reporter

    def _get_test_address_field(self, field_name):
        current_test = self.test
        if current_test is None:
            return None
        return getattr(current_test.__slash__, field_name)


class NullContext(object):

    def __setattr__(self, attr, value):
        raise AttributeError(
            "Cannot set attribute {0!r} on null context".format(attr))

    @property
    def _always_none(self):
        pass

    session = test = test_id = g = internal_globals = \
        test_filename = test_classname = test_methodname = result = _always_none

    reporter = NullReporter()


class _ContextStack(object):

    def __init__(self):
        super(_ContextStack, self).__init__()
        self._stack = [NullContext()]

    def __getattr__(self, attr):
        if not self._stack:
            raise AttributeError(attr)
        return getattr(self._stack[-1], attr)

    def __setattr__(self, attr, value):
        if attr.startswith("_"):
            return super(_ContextStack, self).__setattr__(attr, value)
        setattr(self._stack[-1], attr, value)

    def push(self, ctx):
        self._stack.append(ctx)
        return ctx

    def pop(self):
        assert len(self._stack) != 0
        if len(self._stack) == 1:
            raise RuntimeError("No more contexts to pop")
        return self._stack.pop(-1)

context = _ContextStack()


class ContextAttributeProxy(object):

    def __init__(self, name):
        super(ContextAttributeProxy, self).__init__()
        self._proxy__name = name

    @property
    def _obj(self):
        return getattr(context, self._proxy__name)

    def __getattr__(self, attr):
        return getattr(self._obj, attr)

    def __setattr__(self, attr, value):
        if attr == "_proxy__name":
            return super(ContextAttributeProxy, self).__setattr__(attr, value)
        setattr(self._obj, attr, value)

    def __eq__(self, other):
        return self._obj == other

    def __ne__(self, other):
        return self._obj != other

    def __call__(self, *args, **kwargs):
        return self._obj(*args, **kwargs)  # pylint: disable=not-callable

    def __repr__(self):
        return repr(self._obj)

    def __dir__(self):
        return dir(self._obj)

    __members__ = __dir__

    def __str__(self):
        return str(self._obj)


session = ContextAttributeProxy("session")
test = ContextAttributeProxy("test")
test_id = ContextAttributeProxy("test_id")
g = ContextAttributeProxy("g")
internal_globals = ContextAttributeProxy("internal_globals")
reporter = ContextAttributeProxy("reporter")


def push_context():
    context.push(Context())

def pop_context():
    context.pop()



from slash.utils.pattern_matching import Matcher


def test_string_patterns(suite, config_override):
    selected_test = suite[len(suite) // 2]

    config_override("run.filter_string", selected_test.id)

    for i, test in enumerate(suite):
        if test is not selected_test:
            test.expect_deselect()

    suite.run()


def test_matcher():
    for pattern, matching, not_matching in [
            ('xy', ['blapxy'], ['blap']),
            ('not x', ['blap', 'yyy'], ['blapx', 'xy']),
            ('x and y', ['xy', 'yx', 'x kjfldkjfd y'], ['x', 'y', 'xx']),
            ('(x and some.string) and not z', ['xysome.string', 'some.stringyx'], ['z', 'xysome.stringz']),
            ('exact(x=y)(y=z)', ['exact(x=y)(y=z)'], ['exlact(x=y)(y=z)']),
            ]:

        matcher = Matcher(pattern)
        for string in matching:
            assert matcher.matches(string)
        for string in not_matching:
            assert not matcher.matches(string)



import pytest

from .utils.suite_writer import Suite


def test_run_suite_normal(suite):
    result = suite.run()
    assert result.session.results.get_num_successful() == len(suite)


def test_suite_events(suite):
    test1, test2 = suite[2], suite[3]
    result = suite.run()
    assert result.events['test_start', test1.id].is_before(result.events['test_start', test2.id])





@pytest.fixture
def suite():
    s = Suite()
    for i in range(10):
        s.add_test()
    return s



import colorama
from .utils import TestCase
from slash.utils.color_string import ColorString
from slash.utils.color_string import ColorString, ColorStringBase


class ColoredStringTest(TestCase):

    def test_colored_string(self):
        orig = "some string"
        colored = ColorString(orig, 'red')
        self.assertEquals(str(orig), str(colored))
        self.assertEquals(repr(orig), repr(colored))
        self.assertEquals(
            colored.get_colored(), colorama.Fore.RED + orig + colorama.Fore.RESET)

    def test_get_formatter(self):
        f = ColorString.get_formatter("red")
        self.assert_colored(
            f("test"), colorama.Fore.RED + "test" + colorama.Fore.RESET, "test")

    def test_transformations(self):
        for modifier in [
                (lambda s: s.ljust(20)),
        ]:
            self.assertEquals(modifier(ColorString("string", "red")).get_colored(),
                              ColorString(modifier("string"), "red").get_colored())

    def test_len(self):
        for x in ["test", "me", "here!!!"]:
            self.assertEquals(len(ColorString(x, "red")), len(x))

        self.assertEquals(len("hello" + ColorString(" there", "red")), len("hello there"))

    def test_formatting(self):
        orig = "value1=%s, value2=%s"
        values = (6, 7)
        orig_formatted = orig % values
        colored = ColorString(orig, 'red')
        colored_formatted = colored % values
        self.assert_colored(
            colored_formatted,
            colorama.Fore.RED + orig_formatted + colorama.Fore.RESET,
            orig_formatted
        )
        self.assertIsInstance(colored_formatted, ColorString)
        self.assertEquals(colored_formatted._color, 'red')
        self.assertEquals(colored_formatted._string, orig_formatted)

    def test_concatenation_postfix(self):
        self.assert_colored(
            ColorString('a', color='red') + 'b',
            colorama.Fore.RED + 'a' + colorama.Fore.RESET + 'b',
            'ab'
        )

    def test_concatenation_prefix(self):
        self.assert_colored(
            'a' + ColorString('b', color='red'),
            'a' + colorama.Fore.RED + 'b' + colorama.Fore.RESET,
            'ab'
        )

    def test_complex_concatenation(self):
        self.assert_colored(
            'this is ' +
            ColorString('a message', color='red') + ' to be colored',
            'this is ' + colorama.Fore.RED + 'a message' +
            colorama.Fore.RESET + ' to be colored',
            'this is a message to be colored'
        )

    def assert_colored(self, s, colored, uncolored):
        self.assertIsInstance(s, ColorStringBase)
        self.assertEquals(str(s), str(uncolored))
        self.assertEquals(repr(s), repr(uncolored))
        self.assertEquals(s.get_colored(), colored)



from setuptools import setup, find_packages
import functools
import os
import platform

_PYTHON_VERSION = platform.python_version()
_in_same_dir = functools.partial(os.path.join, os.path.dirname(__file__))

with open(_in_same_dir("slash", "__version__.py")) as version_file:
    exec(version_file.read())  # pylint: disable=W0122

install_requires = [
    "arrow",
    "colorama",
    "confetti>=2.4.1",
    "dessert",
    "emport==1.0.0",
    "gossip>=0.9.0",
    "logbook>=0.4.2",
    "requests>=1.1.0",
    "raven",
    "py>=1.4.20",
    "pyparsing",
    # DO NOT ADD pyforge, lxml or any other package only required for testing
]

if _PYTHON_VERSION < "2.7":
    install_requires.append("argparse")
    install_requires.append("ordereddict")

if _PYTHON_VERSION < "3.3":
    install_requires.append("contextlib2")

setup(name="slash",
      classifiers=[
          "Programming Language :: Python :: 2.6",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3.3",
          "Programming Language :: Python :: 3.4",
      ],
      description="A Testing Framework",
      license="BSD",
      author="Rotem Yaari",
      author_email="vmalloc@gmail.com",
      url="http://vmalloc.github.io/slash",
      version=__version__,  # pylint: disable=E0602
      packages=find_packages(exclude=["tests"]),
      install_requires=install_requires,
      entry_points=dict(
          console_scripts=[
              "slash  = slash.frontend.main:main_entry_point",
          ]
      ),
)



import itertools

from ...exceptions import UnknownFixtures, InvalidFixtureScope, CyclicFixtureDependency

from .namespace import Namespace
from .parameters import get_parametrization_fixtures
from .fixture_base import FixtureBase


_fixture_id = itertools.count()



class Fixture(FixtureBase):

    def __init__(self, store, fixture_func):
        super(Fixture, self).__init__()
        self.fixture_func = fixture_func
        self.info = self.fixture_func.__slash_fixture__
        self.scope = self.info.scope
        self.namespace = Namespace(store, store.get_current_namespace())

    def __repr__(self):
        return '<Function Fixture around {0}>'.format(self.fixture_func)

    def get_value(self, kwargs, active_fixture):
        if self.info.needs_this:
            assert 'this' not in kwargs
            kwargs['this'] = active_fixture
        return self.fixture_func(**kwargs)

    def _resolve(self, store):
        assert self.fixture_kwargs is None

        assert self.parametrization_ids is None
        self.parametrization_ids = []

        kwargs = {}
        parametrized = set()

        for parametrization_fixture in get_parametrization_fixtures(self.fixture_func):
            store.register_fixture_id(parametrization_fixture)
            parametrized.add(parametrization_fixture.name)
            self.parametrization_ids.append(parametrization_fixture.info.id)

        for name in self.info.required_args:
            if name in parametrized:
                continue
            try:
                needed_fixture = self.namespace.get_fixture_by_name(name)

                if needed_fixture.scope < self.scope:
                    raise InvalidFixtureScope('Fixture {0} is dependent on {1}, which has a smaller scope ({2} > {3})'.format(
                        self.info.name, name, self.scope, needed_fixture.scope))

                if needed_fixture is self:
                    raise CyclicFixtureDependency('Cyclic fixture dependency detected in {0}: {1} depends on itself'.format(
                        self.info.func.__code__.co_filename,
                        self.info.name))
                kwargs[name] = needed_fixture.info.id
            except LookupError:
                raise UnknownFixtures(name)
        return kwargs




from contextlib import contextmanager
from .utils.debug import debug_if_needed
from .utils.exception_mark import mark_exception, get_exception_mark
from . import hooks as trigger_hook
from .ctx import context as slash_context
from .conf import config
import functools
import logbook
try:
    import raven # pylint: disable=F0401
except ImportError:
    raven = None
import sys

_logger = logbook.Logger(__name__)

def update_current_result(exc_info):  # pylint: disable=unused-argument
    if slash_context.session is None:
        return
    if slash_context.test is not None:
        current_result = slash_context.session.results.get_result(slash_context.test)
    else:
        current_result = slash_context.session.results.global_result

    current_result.add_exception()

def trigger_hooks_before_debugger(_):
    trigger_hook.exception_caught_before_debugger()  # pylint: disable=no-member

def trigger_hooks_after_debugger(_):
    trigger_hook.exception_caught_after_debugger()  # pylint: disable=no-member

_EXCEPTION_HANDLERS = [
    update_current_result,
    trigger_hooks_before_debugger,
    debug_if_needed,
    trigger_hooks_after_debugger,
    ]

@contextmanager
def handling_exceptions(**kwargs):
    """Context manager handling exceptions that are raised within it

    :param passthrough_types: a tuple specifying exception types to avoid handling, raising them immediately onward
    """
    swallow = kwargs.pop("swallow", False)
    passthrough_types = kwargs.pop('passthrough_types', ())
    try:
        yield
    except passthrough_types:
        raise
    except:
        handle_exception(sys.exc_info(), **kwargs)
        if not swallow:
            raise

def handle_exception(exc_info, context=None):
    """
    Call any handlers or debugging code before propagating an exception onwards.

    This makes sure that the exception can be handled as close as possible to its originating point.

    .. note:: this *DOES NOT* take care of adding the error to the session or test results!
    """
    already_handled = is_exception_handled(exc_info[1])
    msg = "Handling exception"
    if context is not None:
        msg += " (Context: {0})"
    if already_handled:
        msg += " (already handled)"
    _logger.debug(msg, context, exc_info=exc_info if not already_handled else None)

    if not already_handled:
        mark_exception_handled(exc_info[1])
        for handler in _EXCEPTION_HANDLERS:
            handler(exc_info)

def mark_exception_handled(e):
    mark_exception(e, "handled", True)

def is_exception_handled(e):
    """
    Checks if the exception ``e`` already passed through the exception handling logic
    """
    return bool(get_exception_mark(e, "handled", False))

@contextmanager
def get_exception_swallowing_context(report_to_sentry=True):
    """
    Returns a context under which all exceptions are swallowed (ignored)
    """
    try:
        yield
    except:
        if not get_exception_mark(sys.exc_info()[1], "swallow", True):
            raise
        if report_to_sentry:
            capture_sentry_exception()
        _logger.debug("Ignoring exception", exc_info=sys.exc_info())

def noswallow(exception):
    """
    Marks an exception to prevent swallowing by :func:`slash.exception_handling.get_exception_swallowing_context`,
    and returns it
    """
    mark_exception(exception, "swallow", False)
    return exception

def mark_exception_fatal(exception):
    """
    Causes this exception to halt the execution of the entire run.

    This is useful when detecting errors that need careful examination, thus preventing further tests from
    altering the test subject's state
    """
    mark_exception(exception, "fatal", True)
    return exception

def is_exception_fatal(exception):
    return bool(get_exception_mark(exception, "fatal", False))

def disable_exception_swallowing(func_or_exception):
    """
    Marks an exception to prevent swallowing. Can also be used as a decorator around a function to mark all escaped
    exceptions
    """
    if isinstance(func_or_exception, BaseException):
        return noswallow(func_or_exception)
    @functools.wraps(func_or_exception)
    def func(*args, **kwargs):
        try:
            return func_or_exception(*args, **kwargs)
        except BaseException as e:
            disable_exception_swallowing(e)
            raise
    return func

def capture_sentry_exception():
    client = get_sentry_client()
    if client is not None:
        client.captureException()

def get_sentry_client():
    if raven is not None and config.root.sentry.dsn:
        return raven.Client(config.root.sentry.dsn)
    return None



import pytest

from .utils.suite_writer import Suite

@pytest.mark.parametrize('error_adder', ['add_error', 'add_failure'])
def test_adding_errors(error_adder, test_type):

    suite = Suite()
    test = suite.add_test(type=test_type)

    for i in range(2):
        test.append_line('slash.{0}("msg{1}")'.format(error_adder, i))
    test.append_line('slash.{0}(object())'.format(error_adder))

    if error_adder == 'add_error':
        test.expect_error()
    else:
        test.expect_failure()

    results = suite.run().session.results

    [test_result] = results.iter_test_results()

    objs = test_result.get_errors() if error_adder == 'add_error' else test_result.get_failures()
    assert len(objs) == 3

    assert 'msg0' in objs[0].message
    assert 'msg1' in objs[1].message
    assert '<object object at 0x' in objs[2].message


def test_manual_add_error_preserves_traceback(suite, suite_test):
    suite_test.append_line('slash.add_error("error here")')
    suite_test.expect_error()
    summary = suite.run()

    [result] = summary.get_all_results_for_test(suite_test)
    [err] = result.get_errors()

    assert err.traceback is not None

def test_manual_add_error_requires_argument(suite, suite_test):
    suite_test.append_line('slash.add_error()')
    suite_test.expect_error()
    summary = suite.run()
    [result] = summary.get_all_results_for_test(suite_test)
    [err] = result.get_errors()
    assert 'RuntimeError' in str(err)
    assert 'add_error() must be called' in str(err)



from uuid import uuid4

from .element import Element


class Parameter(Element):

    def __init__(self, suite, num_values=3):
        super(Parameter, self).__init__(suite)
        self.name = 'param_{0}'.format(self.id)
        self.values = [str(uuid4()) for i in range(num_values)]

    def write_decorator(self, code_formatter):
        code_formatter.writeln('@slash.parametrize({0!r}, {1!r})'.format(
            self.name, self.values))




import operator

import pytest

from slash._compat import itervalues, reduce


def test_fixtures(suite, suite_test, defined_fixture):
    suite_test.depend_on_fixture(defined_fixture)
    suite.run()


def test_fixture_cleanup_at_end_of_suite(suite):
    fixture = suite.slashconf.add_fixture()
    suite[-1].depend_on_fixture(fixture)
    cleanup = fixture.add_cleanup()

    summary = suite.run()
    assert cleanup in summary.events


def test_fixture_cleanup_failure_fails_test(suite, suite_test, defined_fixture):
    suite_test.depend_on_fixture(defined_fixture)
    cleanup = defined_fixture.add_cleanup(extra_code=['raise Exception()'])
    suite_test.expect_error()
    suite.run()


def test_fixture_parameters(suite, suite_test, defined_fixture):
    defined_fixture.add_parameter()
    suite_test.depend_on_fixture(defined_fixture)
    summary = suite.run()
    all_results = summary.get_all_results_for_test(suite_test)
    num_combinations = reduce(operator.mul, (len(p.values) for p in defined_fixture.get_parameters()))
    assert len(all_results) == num_combinations


def test_fixture_dependency_chain(suite, suite_test):
    fixture1 = suite.slashconf.add_fixture()
    fixture1.add_parameter()
    fixture2 = suite.slashconf.add_fixture()
    fixture2.add_parameter()
    fixture2.depend_on_fixture(fixture1)
    suite_test.depend_on_fixture(fixture2)
    suite.run()


def test_fixture_dependency_both_directly_and_indirectly(suite, suite_test):

    fixture1 = suite.slashconf.add_fixture()
    num_values1 = 2
    fixture1.add_parameter(num_values=num_values1)

    fixture2 = suite.slashconf.add_fixture()
    num_values2 = 3
    fixture2.add_parameter(num_values=num_values2)
    fixture2.depend_on_fixture(fixture1)

    suite_test.depend_on_fixture(fixture1)
    suite_test.depend_on_fixture(fixture2)

    summary = suite.run()
    results = summary.get_all_results_for_test(suite_test)
    assert len(results) == num_values1 * num_values2



# Support fixtures

@pytest.fixture(params=["slashconf", "module"])
def defined_fixture(request, suite, suite_test):
    if request.param == 'slashconf':
        return suite.slashconf.add_fixture()
    elif request.param == 'module':
        return suite_test.file.add_fixture()

    raise NotImplementedError()  # pragma: no cover
